/* 
 Original rotary encoder control program by Richard Visokey, AD7C - www.ad7c.com - revision 2.0 - November 6th, 2013
 
 Arduino rotary encoder code (C)2011 by Ben Buxton (bb@cactii.net) & licenced under GNU GPL version 3
  
 Additional modifications by Robert C. Mazur, VA3ROM http://www.va3rom.com 16 April 2015

 This modification for new WSPR telemery beacon project began on 30 March 2022
 
 Simple JT65/JT9/JT4/FT8/WSPR/FSQ beacon for Arduino, with the Etherkit Si5351A Breakout Board, by Jason Milldrum NT7S. 

 Transmit an abritrary message of up to 13 valid characters (a Type 6 message) in JT65, JT9, JT4, a type 0.0 or type 0.5 FT8 message,
 a FSQ message, or a standard Type 1 message (beacon) in WSPR.

 Connect a momentary push button to pin 12 to use as the transmit trigger. Get fancy by adding your own code to trigger
 off of the time from a GPS or your PC via virtual serial. (** I use rotary encoder so JTEncode library push button code not used **)

 Original code based on Feld Hell beacon for Arduino by Mark Vandewettering K6HX, adapted for the Si5351A by Robert
 Liesenfeld AK6L <ak6l@ak6l.org>.

 Permission is hereby granted, free of charge, to any person obtaining a copy of this software & associated documentation files (the
 "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish,
 distribute, sublicense, &/or sell copies of the Software, & to permit persons to whom the Software is furnished to do so, subject
 to the following conditions:

 The above copyright notice & this permission notice shall be included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE & NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR
 ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

/************************************************************************************************************
 *                                             ARDUINO INCLUDE FILES                                        *
 ************************************************************************************************************/
#include <rs_common.h>
#include <int.h>
#include <string.h>

#include <Wire.h>                             // required Wire language support library (supplied with Arduino IDE)
#include <LiquidCrystal_I2C.h>                // required LCD I2C support library (supplied with Arduino IDE)
#include "cactus_io_BME280_I2C.h"             // Add-on BM280 sensor module library by Cactus.io http://cactus.io/ (Under "Hookups BME280")
#include "RTClib.h"                           // Add-on DS3231 RTC module library by Adafruit https://github.com/adafruit/RTClib
#include <JTEncode.h>						              // JT65/JT9/JT4/FT8/WSPR/FSQ https://tinyurl.com/yckuxsbx
#include <si5351.h>                           // Si5351 DDS support library

/*************************************************************************************************************
 *                               USER GLOBAL PROGRAM PARAMETERS (CHANGE AS REQUIRED)                         * 
 *************************************************************************************************************/

// KY-40 rotary encoder definitions (if used)
// set to "true" to enable Arduino's internal "weak" pullup resistors for rotary encoders that don't have their own added pullup resistors (else set to "false")
#define ENABLE_PULLUPS false

// set to "true" for half-step emcoders like KY-40 with its own active electronics built-in (else set to "false")
#define ENABLE_HALFSTEP true

#define ROTARY_PIN1 17                        // rotary encoder input pins connected to Arduino analog input pins A2 & A3 now used for digital input (reverse if tuning direction opposite)
#define ROTARY_PIN2 16

#define ROTARY_BUTTON 15                      // rotary encoder shaft push button switch connected to Arduino's analog input pin A1 now used for digital input

// rotary encoder possible directions CCW = counter-clockwise & CW = clockwise
#define DIR_CCW 0x10                
#define DIR_CW 0x20

// lookup table used to encode telemetry call sign positions 2 to 6 ("0", "1" or "Q" used for call sign position 1 denotes telemetry beacon or non-Amateur Radio call sign)
char lookup_table[36] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";

byte index = 0;                               // lookup table index pointer

// comment/uncomment WSPR band definitions as required & recompile
// WSPR 10 m band definitions
//#define VFO_LOWER 28126000UL                  // DDS/VFO lower frequency limit 10 m WSPR band
//#define VFO_CENTR 28126100UL                  // DDS/VFO centre frequency 10 m WSPR band
//#define VFO_UPPER 28126194UL                  // DDS/VFO upper frequency limit 10 m WSPR band (5 Hz below band edge)

// WSPR 20 m band definitions
//#define VFO_LOWER 14097000UL                  // DDS/VFO lower frequency limit 20 m WSPR band
//#define VFO_CENTR 14097100UL                  // DDS/VFO centre frequency 20 m WSPR band
//#define VFO_UPPER 14097194UL                  // DDS/VFO upper frequency limit 20 m WSPR band (5 Hz below band edge)

// WSPR 22 m band definitions
#define VFO_LOWER 13555300UL                  // DDS/VFO lower frequency limit 22 m WSPR band
#define VFO_CENTR 13555400UL                  // DDS/VFO centre frequency 22 m WSPR band
#define VFO_UPPER 13555494UL                  // DDS/VFO upper frequency limit 22 m WSPR band (5 Hz below band edge)

// WSPR 30 m band definitions
//#define VFO_LOWER 10140100UL                  // DDS/VFO lower frequency limit 30 m WSPR band
//#define VFO_CENTR 10140200UL                  // DDS/VFO centre frequecy 30 m WSPR band
//#define VFO_UPPER 10140294UL                  // DDS/VFO upper frequency limit 30 m WSPR band (5 Hz below band edge)

// WSPR 40 m band definitions
//#define VFO_LOWER 7040000UL                   // DDS/VFO lower frequency limit 40 m WSPR band
//#define VFO_CENTR 7040100UL                   // DDS/VFO centre frequency 40 m WSPR band
//#define VFO_UPPER 7040194UL                   // DDS/VFO upper frequency limit 40 m WSPR band (5 Hz below edge)

// BME280 sensor module correction values (obtained from personal weather station (PWS) or airport weather reports)
// in millibars/hectopascals (divide by 10 for kilo Pascals (kPa) ie. 1004 mB = 100.4 kPa)
//const float bme_MB_corr = 25.75;              // home pressure correction
const float bme_MB_corr = 51.75;              // camp pressure correction
const float bme_TP_corr = -2.75;              // in degrees Celsius
const float bme_RH_corr = 2.00;               // in percentage

// BME280 sensor module working variables
float bme_TP;							
float bme_RH;
float bme_DP;
float bme_MB;

float voltage;           					            // analog voltage (3:1) divider working variable

#define TIMER_INTERVAL 60000                  // time in milliseconds (60 seconds) for reading BME280 & voltage divider sensor values
 
#define TONE_SPACING 146                      // WSPR frequency shift (1.46 Hz) per WSPR symbol 0 to 3 = 0, 1.465, 2.930 & 4.395 Hz)
#define TONE_DELAY 683                        // transmit time length for each WSPR symbol in milliseconds
#define SYMBOL_COUNT 162                      // number of WSPR symbols to transmit
#define CORRECTION 0                          // change this for frequency correction in parts-per-billion

//#define DEFAULT_FREQ VFO_CENTR              // default WSPR frequency
#define DEFAULT_FREQ 13555460UL
 
uint8_t WSPR_buffer[SYMBOL_COUNT];

// WSPR propagation & telemetry beacon working variables (change call[], loc[] & dbm as req'd)
char call_telem[] = "1A0AAA";				// default WSPR telemetry beacon call sign buffer
char call[] = "1A0AAA";"		            // default WSPR propagation beacon call sign buffer
char loc[] = "AA00AA";					    // default WSPR propagation/telemetry beacon 4-character Maidenhead grid square (fixed position)
uint8_t dbm = 7;							// default WSPR propagation/telemetry beacon power in decibel-milliwatts (dBm) Here 7 dBm = 5 milliwatts (mW)

/*************************************************************************************************************
 *                                         PROGRAM GLOBAL USE VARIABLES                                      *
 *************************************************************************************************************/
RTC_DS3231 rtc;							      // create real-time-clock (RTC) object

JTEncode jtencode;						      // create JTEncode object

Si5351 si5351;                                // create Si5351 object

BME280_I2C bme(0x76);                         // create BME280 sensor module class & define I2C hex address 0x76 (cactus.io library allows address reprogramming)

unsigned long freq = DEFAULT_FREQ;            // default starting AD9850 DDS VFO frequency (centre of WSPR band used)
byte increment = 1;                           // fixed AD9850 DDS VFO frequency tuning step/increment (1 Hz) don't need it larger (200 Hz wide band)

unsigned long timepassed = millis();          // variable for Arduino elapsed millisecond timer (value since program start used for sensor reading timing)

// byte (individual) digital frequency place holder variables (used for LCD frequency display)
byte ones, tens, hundreds, thousands, tenthousands, hundredthousands, millions;

volatile unsigned char state = 0;             // present KY-40 rotary encoder's state (pseudo interrupt)

/* rotary encoders generate 2-bit Gray code (a quadrature signal) as they rotate clockwise or counter-wise
   "half-step" rotary encoders generate two Gray code state signals per rotation between click-stops (i.e. number of discrete steps/increments per 360 degrees of rotation)
   "full-step" rotary encoders generate one Gray code state signals per rotation between click-stops
   "continous turn" rotary encoders generate continous Gray state signals as they turn because they have no click-stops (not suited for our purpose here)
   
                          Gray Code Logic Table
                        Position  Bit 1    Bit 2
                        ------------------------
                           0        0        0
                          1/4       1        0
                          1/2       1        1
                          3/4       0        1
                           1        0        0

   Gray code table creates a "state machine" & eliminate any confusion between state changes (i.e. turning from "11" can only generate "10" or "01" anything else is
   rejected as erroneous or "illogical") which provides for built-in mechanical "debounce" of rotary encoder & increases response speed 
   therefore no bypass/switch noise surpression capacitors required 
*/                       

#if ENABLE_HALFSTEP
  
  // use half-step state table (emits a code at 00 & 11) for KY40 rotary encoders (on PCB with ancillary components)
  const unsigned char ttable[6][4] = {
    {0x3 , 0x2, 0x1,  0x0}, {0x23, 0x0, 0x1,  0x0},
    {0x13, 0x2, 0x0,  0x0}, {0x3 , 0x5, 0x4,  0x0},
    {0x3 , 0x3, 0x4, 0x10}, {0x3 , 0x5, 0x3, 0x20}
  };

#else

  // use the full-step state table (emits a code at 00 only) so uncomment for "naked" rotary encoders (no PCB with ancillary components)
  const unsigned char ttable[7][4] = {
  {0x0, 0x2, 0x4, 0x0}, {0x3, 0x0, 0x1, 0x10},
  {0x3, 0x2, 0x0, 0x0}, {0x3, 0x2, 0x1, 0x0},
  {0x6, 0x0, 0x4, 0x0}, {0x6, 0x5, 0x0, 0x20},
  {0x6, 0x5, 0x4, 0x0}
  };

#endif

// define LCD class & I2C driver pack address 0x27 using 16 character x 2 line display
// I2C driver packe connected to Arudino Nano analog pins A4 (SDA) & A5 (SLC)
// BME sensor & DS3231 RTC also connect to same pins (multiple I2C devices can share same lines because they have individual/different I2C addresses
LiquidCrystal_I2C lcd(0x27, 16, 2);        

/*************************************************************************************************************
 *                                                PROGRAM FUNCTIONS                                          *
 *************************************************************************************************************/

/*
From "A Module 40 Meter CW Transciever with VFO" (custom daughterboard) QST March 2016 (see RM056 supplement file)
This method is used to read the value from the R7, R8, C7 voltage divider circuit going to pin 26/A7 of the Arduino Nano,
process the required calculations, then display the voltage on the LCD by Hank Ellis K5HDE
V1.0 14 June 2016 Implementation of the basic voltmeter
V1.1 23 June 2016 Fixed bug that blanks the voltmeter when the frequency step is changed via the encoder pushbutton

VA3ROM modifications for averaging voltage to improve accurracy June 2022
*/

// read analog voltage divider & calculate current battery/solar voltage level
void read_Voltmeter() { 

  int avg_volts = 0;
  
  float temp = 0.0;                       	  // temporary number used in calculations
  float r7 = 20000.0;                     	  // value of R7 (use 20,000 ohms if possible)
  float r8 = 8000.0;                      	  // arbitrary value assigned to R8 (10k potentiometer)

  for (byte i = 0; i < 10; i++) {             // take average of 10 voltage readings
 
    avg_volts = avg_volts + analogRead(A0);   // read the value from the voltage divider on pin A0 (determined by actual R8 resistance)
  
    delay(10);
  
  }

  avg_volts = avg_volts / 10;
  
  temp = (avg_volts * 5.0) / 1024.0;          // convert it to a usable number (assumes Vcc actual is 5.0 VDC -- usually not but good enough for purpose)
  
  voltage = temp / (r8 / (r7 + r8)); 		      // standard formula of a voltage divider circuit
	voltage = voltage * 10;					            // remove decimal point i.e. 12.5 = 125 for LCD display (saves display space)
  
  lcd.print(round(voltage));
  
} // end read_Voltmeter()

// initialize rotary encoder (call this once from setup() function)
void rotary_init() {

  // define rotary encoder pins for input use
  pinMode(ROTARY_PIN1, INPUT_PULLUP);
  pinMode(ROTARY_PIN2, INPUT_PULLUP);

  pinMode(ROTARY_BUTTON, INPUT_PULLUP);       // rotary encoder shaft push button connected to Ardunio Uno analog input pin A1
    digitalWrite(ROTARY_BUTTON, HIGH);        // set value to high (open or not pushed/pressed)

  #ifdef ENABLE_PULLUPS

    // for rotary encoders that don't have own external pullup resistors
    digitalWrite(ROTARY_PIN1, HIGH);
    digitalWrite(ROTARY_PIN2, HIGH);

  #endif

} // end rotary_init()

// read rotary encoder input pins & process as required (call this from inside loop or interrupt (e.g. pin change or timer)
// returns 0 on no event, otherwise 0x80 or 0x40 depending on the rotary encoder's direction of rotation
unsigned char rotary_process() {

  // get rotary encoder's two pin states as it turns, merge them & lookup result in Gray code table
  unsigned char pinstate = (digitalRead(ROTARY_PIN2) << 1) | digitalRead(ROTARY_PIN1);  
    state = ttable[state & 0xf][pinstate];

  return (state & 0x30);                   	  // send result back to calling routine

} // end rotary_process()

// break up frequency to byte (individual digit) segments & display on LCD
void showFrequency(){

  millions = int(freq / 1000000);
  hundredthousands = (freq / 100000) % 10;
  tenthousands = (freq / 10000) % 10;
  thousands = (freq / 1000) % 10;
  hundreds = (freq / 100) % 10;
  tens = (freq / 10) % 10;
  ones = (freq / 1) % 10;
  
  // set LCD cursor to home position & clear frequency displayed 
  lcd.setCursor(0,0); lcd.print("                ");

  // resposition cursor as required for new frequency & display each frequency byte (individual digit) segment
  if (millions > 9)
    lcd.setCursor(1, 0);
  else
    lcd.setCursor(2, 0);
    
  lcd.print(millions);
  lcd.print(".");
  lcd.print(hundredthousands);
  lcd.print(tenthousands);
  lcd.print(thousands);
  lcd.print(".");
 
  lcd.print(hundreds);
  lcd.print(tens);
  lcd.print(ones);
 
  lcd.print(" MHz  ");

} // end showFrequency()

// setup additional program runtime parameters
void setup() {

  bool i2c_found;

  Wire.begin();                               // start Arduino Wire library functions (mainly I2C)

  rotary_init();                              // initialize rotary encoder          

  lcd.init();                                 // initialize LCD display, set backlight & home LCD cursor
  lcd.backlight(); 
  lcd.setCursor(0, 0); 

  // Initialize the Si5351 change the 2nd parameter in init if using a ref osc other than 25 MHz (TCXO default for QRP Labs Arduino DDS transmitter shield)
  i2c_found = si5351.init(SI5351_CRYSTAL_LOAD_8PF, 0, CORRECTION); 
  
  if(!i2c_found) {                            // check status of Si5351 DDS
    
    lcd.print("Can't find DDS!");
  
    while(1);
    
  }

  if (!rtc.begin()) {                         // check status of DS3231 RTC module
 
      lcd.print("Can't find RTC!");           // display on LCD keypad shield
    
    while(1);
    
  }
   
  if (!bme.begin()) {                         // check status of BME280 sensor module
    
      lcd.print("Can't find BME280!");        // display on LCD keypad shield
    
    while (1);
  
  }  

  showFrequency();                            // display default WSPR/DDS frequency

  // set CLK0 output RF curent drive (can adjust RF power out using QRP Labs Arduino DDS transmitter shield R1 4.7k potentiometer)
  si5351.drive_strength(SI5351_CLK0, SI5351_DRIVE_8MA); // set CLK0 ouptut to maximum drive  
     si5351.output_enable(SI5351_CLK0, 0);    // turn off CLK0 RF output
     
  read_Sensor();							                // get BME280 environmental sensor readings
  
  read_Voltmeter();							              // get analog voltage divider reading

  build_WSPR_bcn();							              // build WSPR type 1 propagation beacon


} // end function setup()

// gets BME280 environmental sensor readings
void read_Sensor() {

  bme.readSensor();                         // get BME280 sensor module temperature, relative humidity & barometric pressure (digital not analog readings)
    delay(10);
  
  lcd.setCursor(0, 1);                      // home LCD cursor to second display line
   
  // break down & correct various BME280 sensor module readings
  bme_TP = bme.getTemperature_C() + bme_TP_corr;
    
  bme_RH = bme.getHumidity() + bme_RH_corr;
    
  bme_DP = bme_TP - (100 - bme_RH) / 5;	    // calculate dew point based on temperature & relative humidity (fast but less accurate but good enough for purpose)
  
  bme_MB = bme.getPressure_MB() + bme_MB_corr;

	// old trick were negative temperatures are made positive & have 50 added (i.e. -25 = 25 + 50 = "75", when you see this you subract 50 & negate result)
	if (bme_TP < 0)                           // handle below 0 temps
      bme_TP = (-1 * bme_TP) + 50;
      
  if (bme_DP < 0)                           // handle below 0 temps
      bme_DP = (-1 * bme_DP) + 50;
  
	// display temperature, dew point & relative humidity on LCD 
  print_Digits(round(bme_TP)); lcd.print(' '); 
    print_Digits(round(bme_DP)); lcd.print(' '); 
      print_Digits(round(bme_RH)); lcd.print(' ');

  if (bme_MB >= 1000) {                      // baro >= 1000 MB we only want last 3 digits (i.e. 1034 mB = "034") can't have presure > 1099 mB on Earth!)

    bme_MB = bme_MB - 1000;                  // adjust pressure for LCD display
      lcd.print('0');                        // reposition LCD cursor for pressures less than 1000 millibars (we don't use leading "1" if > 1000 mB)
  }
      
  if (bme_MB > 0) {

      print_Digits(round(bme_MB));           // display barometeric pressure on LCD
        lcd.print(' ');

   }
   else
      lcd.print("00 ");                       // add more padding if pressure exactly 1000 mB

  
} // end read_Sensor()
  
// main program begins here 
void loop() {

  // was rotary shaft button pressed? (used to start timeing/transmission sequence
  if (digitalRead(ROTARY_BUTTON) == LOW) {

    // set to default program values (don't use anything but seconds)
	  rtc.adjust(DateTime(2000, 1, 1, 0, 0, 0));
    
    WSPR_bcn();								    // build WSPR type 1 propagation beacon & transmit
    
  }
  
  // check to see if rotary encoder has been rotated or not
  unsigned char result = rotary_process();
  
  if (result) {                         	  // value return from rotary encoder?
 
    if (result == DIR_CW)                	  // rotary encoder shaft rotated clockwise direction (DIR_CW)
      freq = freq + increment;                // increase frequency by tuning step/increment value
    else                               	  	  // other option is counter clockwise direction (DIR_CCW)
      freq = freq - increment;                // decrease frequency by tuning step/increment value
   
    // circular DDS/VFO buffer rolls around between upper & lower DDS/VFO limits
    if (freq > VFO_UPPER)                 	  // can't exceed upper DDS/VFO frequnecy limit
      freq = VFO_LOWER;                      
    else if (freq < VFO_LOWER)           	  // can't go below lower DDS/VFO frequency limit  
      freq = VFO_UPPER;                    

   showFrequency();                   		  // display AD98500/VFO frequency
  
  }
 
  // time to get sensor & voltage values?
  if ((millis() - timepassed) > TIMER_INTERVAL) { 
         
    timepassed = millis();                    // reset time to next checks

    read_Sensor();                        
      read_Voltmeter();
    
  }

} // end loop() function

// Print/display digits on CD with leading 0 padding (if required)
void print_Digits(int digits) { 
    
  if(digits < 10) 
     lcd.print('0');
  
  lcd.print(digits);
    
}// end print_Digits()

// build WSPR type 1 propagation beacon
void build_WSPR_bcn() {
 
  // clear transmit buffer
  memset(WSPR_buffer, 0, SYMBOL_COUNT);			// clear transmit buffer & encode WSPR type 1 propagation beacon
     
  jtencode.wspr_encode(call, loc, dbm, WSPR_buffer);
      
}// end built_WSPR_bcn()

// build WSPR type 1 telemetry beacon (only call sign encoded, grid square & dBm remove same)
void build_WSPR_telem_bcn() {
 
  memset(WSPR_buffer, 0, SYMBOL_COUNT);		   // clear transmit buffer & encode WSPR type 1 propagation beacon

  jtencode.wspr_encode(call_telem, loc, dbm, WSPR_buffer);

}// end built_WSPR_bcn()

// determines if we should build WSPR type 1 propagation or telemetry beacon
void WSPR_bcn() {

  byte tx_flag = 0;                           // transmit flag 0 = transmit WSPR type 1 propagation beacon, flag = 2 = transmit WSPT type 1 telemetry beacon

  do {
    
    lcd.setCursor(0,0); lcd.print("  ...WAITING... ");

    do {
  
        DateTime now = rtc.now();             //get current DS3231 RTC mdule date & time
    
      // transmit WSPR type 1 propagation(tx_flag = 0) or telemetry (tx_flag =2 ) beacon every 10 minutes (modulo 10) or 6 times an hour
      if ((now.minute() % 10) == tx_flag) break; 
      
    } while (1 == 1);                         // stay in timing loop until next 10 minute transmission block (0, 10, 20, 30, 40 & 50 minutes)

    if (tx_flag == 0) {                       // transmitting in first even minute (dislay either WSPR type 1 propagation or telemetry beacon information on LCD)

      // time to get sensor readings (refresh after WSPR propagation & telemetry beacons transmitted)
      if ((millis() - timepassed) > TIMER_INTERVAL) { 
         
        timepassed = millis();                //if the time since last update has elapsed 

        read_Sensor();
          
        read_Voltmeter();
    
      }
    
      lcd.setCursor(0,0); lcd.print(call); lcd.print(' '); lcd.print(loc); lcd.print(' '); 
      
      if (dbm < 10) 
        lcd.print(' ');
      
      lcd.print(dbm); lcd.print(" =");

      tx_WSPR_bcn();                          // transmit WSPR type 1 propagation beacon first (already encoded)
  
      encode_WSPR_telem_bcn();                // encode WSPR type 1 telemetry beacon in the 10 seconds to next even minute

      build_WSPR_telem_bcn();                 // build WSPR type 1 telemetry beacon

      tx_flag = 2;                            // transmitting in second even minute
      
    }  
    else {                                    // tx_min flag determines if we need to rebuilt WSPR type 1 propagation beacon & reset tx_min flag

      lcd.setCursor(0,0); lcd.print(call_telem); lcd.print(' '); lcd.print(loc); lcd.print(' '); 

      if (dbm < 10) 
        lcd.print(' ');
      
      lcd.print(dbm); lcd.print(" =");
          
      tx_WSPR_bcn();

      build_WSPR_bcn();
      
      tx_flag = 0;                            // set back to WSPR type 1 propagation beacon

    }

  } while (1 == 1);
      
}// end WSPR_bcn()

// transmit either WSPR type 1 propagation or telemetry beacon
void tx_WSPR_bcn() {

  si5351.output_enable(SI5351_CLK0, 1);        // turn on CLK0 RF output
      
  for (uint8_t i = 0; i < SYMBOL_COUNT; i++) { // transmit WSPR symbols
    
    si5351.set_freq((freq * 100) + (WSPR_buffer[i] * TONE_SPACING), SI5351_CLK0);
      
    delay(TONE_DELAY);

  }

  si5351.output_enable(SI5351_CLK0, 0);       // turn off CLK0 RF output
  
}// end tx_WSPR_bcn()

// encode sensor values into WSPR type 1 telemetry beacon
// this is a brute force so cascaded if/else if method until I figure out a better way (see RM056 supplement file)
void encode_WSPR_telem_bcn() {

  if (bme_TP >= 50)                           // if required, restore original negative temperature value
    bme_TP = 50 - bme_TP;

  if (bme_TP < -10)          							  // temperature valid range check
	  bme_TP = -10;
  if (bme_TP > 40)
    bme_TP = 40;
	
  index = map(bme_TP, -10, 40, 10, 35);
    call_telem[1] = lookup_table[index];      // encode air temperature

  index = map(bme_RH, 0, 100, 0, 9);          // encode relative humidity
    call_telem[2]= lookup_table[index];

  if (bme_DP >= 50)                           // if required, restore original negative temperature value                      
    bme_DP = 50 - bme_DP;

  if (bme_DP < -10)						            	  // dewpoint valid range check
	  bme_DP = -10;
  if (bme_DP > 40)
	  bme_DP = 40;

  index = map(bme_DP, -10, 40, 10, 35);
    call_telem[3] = lookup_table[index];      // encode dew point

  if (bme_MB < 100)                           // correct for pressures >= 1000 mB (ro store 1000's digit)
    bme_MB = bme_MB + 1000;

  if (bme_MB < 975)           							  // pressure valid range check
	  bme_MB = 975;
  if (bme_MB > 1050)
	  bme_MB = 1050;
  
  index = map(bme_MB, 975, 1050, 10, 35);     // encode air pressure
    call_telem[4] = lookup_table[index];

  if (voltage < 100)						              // voltage valid range check
    voltage = 100;
  if (voltage > 150)
	  voltage = 150;

  index = map(voltage, 100, 150, 10, 35);     // encode battery voltage (multipled by 10 earlier to shift decimal point (also map function only works with integer values)
    call_telem[5] = lookup_table[index];
 
}// end encode_WSPR_telem_bcn()
